#!/bin/env python

import math
import porterstemmer
import stoplist
import sys

po = porterstemmer.PorterStemmer()
sl = stoplist.StopList()

def find_sentence(line, begin):
    end = len(line)
    p = begin
    while p < end:
        q = line.find('. ', p)
        if q == -1:
            return (begin, end)
        else:
            q += 2
            if end <= q:
                return (begin, end)
            elif line[q].isupper() or line[q] == ' ':
                return (begin, q)
            p = q
    return (begin, end)

def split_sentences(text):
    begin = 0
    sentences = []
    while 1:
        begin, end = find_sentence(text, begin)
        sentences.append(text[begin:end])
        if end == len(text):
            break
        else:
            begin = end
    return sentences

def tokenize(sentence):
    sentence = sentence.replace('"', ' ')
    tokens = sentence.lower().replace('-', ' ').rstrip(' .?!').split(' ')
    tokens = [token.strip('();:.,?!') for token in tokens if token]
    tokens = [po.do(token) for token in tokens if token not in sl]
    return tokens

if __name__ == '__main__':
    fi = sys.stdin
    fo = sys.stdout

    n = 0
    dftbl = {}
    tftbl = {}
    contents = []
    for line in fi:
        line = line.strip()
        if line.startswith('<MedlineCitation'):
            pmid = ''
            unique_tokens = set()
            n += 1
        elif line.startswith('<PMID>') and not pmid:
            pmid = line[6:-7]
        elif line.startswith('<AbstractText>'):
            text = line[14:-15]
            sentences = split_sentences(text)
            for i in range(len(sentences)):
                sid = '%s:%d' % (pmid, i)
                sentence = sentences[i]
                tokens = tokenize(sentence)
                unique_tokens.update(tokens)
                for token in tokens:
                    tftbl.setdefault(token, 0)
                    tftbl[token] += 1
                contents.append((sid, sentence, tokens))
            
            for token in unique_tokens:
                dftbl.setdefault(token, 0)
                dftbl[token] += 1

    for sid, sentence, tokens in contents:
        if len(sentence) < 20:
            continue
        fo.write('# %s %s\n' % (sid, sentence))
        fo.write('"%s"\t%d' % (sid, len(sentence)))
        for token in tokens:
            weight = tftbl[token] * math.log(n / float(dftbl[token]))
            weight /= math.log(len(tokens))
            fo.write('\t"%s":%f' % (token, weight))
        fo.write('\n')
